/*
*  Rubrica
*  file: adb.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <gtk/gtk.h>
#include <libral.h>

#include "adb.h"
#include "models.h"
#include "types.h"
#include "gui_types.h"
#include "callbacks.h"

/*    signals enumeration 
 */
enum {
  RENAMED,
  LAST_SIGNAL 
};


static guint rubrica_adb_signals[LAST_SIGNAL] = {0};


struct _RubricaAdbPrivate {
  RubricaAbook* adb;
  RubricaGroupSet* user_set;     // user defined groups for this addressbook
  RubricaGroupSet* set;          // default groups + user defined groups

  GtkTreeView* tree;

  gboolean dispose_has_run;
};


static void rubrica_adb_class_init (RubricaAdbClass* klass);
static void rubrica_adb_init       (RubricaAdb* obj);

static void rubrica_adb_dispose    (RubricaAdb* obj);
static void rubrica_adb_finalize   (RubricaAdb* obj);



GType
rubrica_adb_get_type()
{
  static GType adb_type = 0;
  
  if (!adb_type)
    {
      static const GTypeInfo adb_info =
	{
	  sizeof(RubricaAdbClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_adb_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaAdb),
	  0,
	  (GInstanceInitFunc) rubrica_adb_init
	};

      adb_type = g_type_register_static (G_TYPE_OBJECT, "RubricaAdb",
					 &adb_info, 0);
    }
  
  return adb_type;
}


static void
rubrica_adb_class_init(RubricaAdbClass* klass)
{
  GObjectClass *class; 

  class           = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_adb_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_adb_finalize;

  
  /* class signals 
   */ 
  rubrica_adb_signals[RENAMED] =
    g_signal_new("renamed", 
		 RUBRICA_ADB_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAdbClass, renamed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_STRING);         /* params type */
}


static void
rubrica_adb_init(RubricaAdb* self)
{
  self->private = g_new(RubricaAdbPrivate, 1);

  /*    the addressadb
  */
  self->private->adb = rubrica_abook_new(); 
 
  /*    addressbook's categories
   */
  self->private->user_set  = rubrica_group_set_new();
  self->private->tree      = NULL;

  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_adb_dispose (RubricaAdb* self)
{
  g_return_if_fail(IS_RUBRICA_ADB(self));
  
  if (self->private->dispose_has_run)
    return;

  if (self->private->adb)
    rubrica_abook_free(self->private->adb);
  

  g_object_unref(self->private->tree);
  
  self->private->adb      = NULL;
  self->private->tree     = NULL;
  
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_adb_finalize (RubricaAdb* self)
{
  g_return_if_fail(IS_RUBRICA_ADB(self));
  
  g_free(self->private);
  self->private = NULL;
}


/*    Public
*/

/**
 * rubrica_adb_new
 * @set: a #RubricaAdb
 *
 * create a new #RubricaAdb
 *
 * returns: a new allocated #RubricaAdb*
 */
RubricaAdb*
rubrica_adb_new (void)
{
  RubricaAdb* adb;

  adb = g_object_new(rubrica_adb_get_type(), NULL);

  return adb;
}


/**
 * rubrica_adb_free
 * @set: a #RubricaAdb
 *
 * free memory owned by a #RubricaAdb
 */
void
rubrica_adb_free(RubricaAdb* adb)
{
  g_return_if_fail(IS_RUBRICA_ADB(adb));

  g_object_unref(adb);   
}


RubricaAbook* 
rubrica_adb_get_addressbook(RubricaAdb* adb)
{
  g_return_val_if_fail(IS_RUBRICA_ADB(adb), NULL);
  
  return adb->private->adb;
}


GtkTreeView*
rubrica_adb_get_tree(RubricaAdb* adb)
{
  g_return_val_if_fail(IS_RUBRICA_ADB(adb), NULL);

  return adb->private->tree;
}


RubricaGroupSet* 
rubrica_adb_get_user_groups(RubricaAdb* adb)
{
  g_return_val_if_fail(IS_RUBRICA_ADB(adb), NULL);

  return adb->private->user_set;
}


RubricaGroupSet* 
rubrica_adb_get_groups(RubricaAdb* adb)
{
  g_return_val_if_fail(IS_RUBRICA_ADB(adb), NULL);

  return adb->private->set;
}


gchar* 
rubrica_adb_get_name (RubricaAdb* adb)
{
  g_return_val_if_fail(IS_RUBRICA_ADB(adb), NULL);

  return rubrica_abook_get_name(RUBRICA_ABOOK(adb->private->adb));
}

void
rubrica_adb_set_tree (RubricaAdb* adb, GtkTreeView* tree)
{
  g_return_if_fail(IS_RUBRICA_ADB(adb));
  
  adb->private->tree = tree;
}


void
rubrica_adb_set_name (RubricaAdb* adb, gchar* name)
{
  g_return_if_fail(IS_RUBRICA_ADB(adb));
  
  rubrica_abook_set_name(RUBRICA_ABOOK(adb->private->adb), name);
  g_signal_emit_by_name(G_OBJECT(adb), "renamed", name, G_TYPE_STRING);
}

void
rubrica_adb_set_user_groups (RubricaAdb* adb, RubricaGroupSet* set)
{
  g_return_if_fail(IS_RUBRICA_ADB(adb));
  g_return_if_fail(IS_RUBRICA_GROUP_SET(set));

  adb->private->user_set = set;
}


void 
rubrica_adb_set_groups (RubricaAdb* adb, RubricaGroupSet* set)
{
  g_return_if_fail(IS_RUBRICA_ADB(adb));
  g_return_if_fail(IS_RUBRICA_GROUP_SET(set));
  
  adb->private->set = set;
}
